<?php

namespace Pinpoint\ProductPersonalisation\Controller\Preview;

use function GuzzleHttp\Psr7\str;
use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Framework\App\Filesystem\DirectoryList;

/**
 * Class GenerateImage
 * @package Pinpoint\ProductPersonalisation\Controller\Preview
 */
class GenerateImage extends Action
{
    /**
     * @var \Magento\Framework\Controller\Result\JsonFactory
     */
    protected $resultJsonFactory;

    /**
     * @var \Magento\Catalog\Model\ProductFactory
     */
    protected $product;

    /**
     * @var \Magento\Framework\Filesystem
     */
    protected $fileSystem;

    /**
     * @var \Pinpoint\ProductPersonalisation\Model\ImageTemplate
     */
    protected $imageTemplate;

    /**
     * @var \Pinpoint\ProductPersonalisation\Model\ImageRow
     */
    protected $imageRow;

    /**
     * @var \Pinpoint\ProductPersonalisation\Model\Font
     */
    protected $font;

    /**
     * @var string
     */
    const PREVIEW_CACHE_FOLDER = 'pinpoint_pp_previews';

    /**
     * @var string
     */
    const FONTS_FOLDER = 'pinpoint_fonts';

    /**
     * GenerateImage constructor.
     * @param Context $context
     * @param \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory
     * @param \Magento\Catalog\Model\ProductFactory $product
     * @param \Magento\Framework\Filesystem $fileSystem
     * @param \Pinpoint\ProductPersonalisation\Model\ImageTemplate $imageTemplate
     * @param \Pinpoint\ProductPersonalisation\Model\ImageRow $imageRow
     * @param \Pinpoint\ProductPersonalisation\Model\Font $font
     */
    public function __construct(
        Context $context,
        \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory,
        \Magento\Catalog\Model\ProductFactory $product,
        \Magento\Framework\Filesystem $fileSystem,
        \Pinpoint\ProductPersonalisation\Model\ImageTemplate $imageTemplate,
        \Pinpoint\ProductPersonalisation\Model\ImageRow $imageRow,
        \Pinpoint\ProductPersonalisation\Model\Font $font
    ) {
        $this->resultJsonFactory = $resultJsonFactory;
        $this->product = $product;
        $this->fileSystem = $fileSystem;
        $this->imageTemplate = $imageTemplate;
        $this->imageRow = $imageRow;
        $this->font = $font;
        parent::__construct($context);
    }

    /**
     * @return \Magento\Framework\Controller\Result\Json
     */
    public function execute()
    {
        $resultJsonFactory = $this->resultJsonFactory->create();
        $product = $this->product->create();

        if ($productSku = $this->getRequest()->getParam('sku')) {
            $product = $product->loadByAttribute('sku', $productSku);
            $options = $this->extractOptions($this->getRequest()->getParam('options'));
            $mediaDirectory = $this->fileSystem->getDirectoryRead(DirectoryList::MEDIA);

            $cacheImagePath =
                $mediaDirectory->getAbsolutePath()
                . self::PREVIEW_CACHE_FOLDER
                . DIRECTORY_SEPARATOR
                . md5($productSku . json_encode($options)) . '.png'
            ;

            if (\file_exists($cacheImagePath)) {
                return $resultJsonFactory->setData(['image' => \base64_encode(\file_get_contents($cacheImagePath))]);
            }

            $resultJsonFactory->setData(['image' => '']);

            if ($product->getId()) {
                $basePath = $this->fileSystem->getDirectoryRead(DirectoryList::ROOT)->getAbsolutePath();
                $data = $this->getTemplateByRowCount($productSku, $this->getLineTotal($options));
                $im = new \Imagick($basePath . $data['template']['template_image']);

                foreach ($options as $key => $option) {
                    if (substr($key, 0, 4) == 'line') {
                        $draw = new \ImagickDraw();
                        $lineNumber = explode("_", $key)[1];

                        $row = $this->getTemplateRowArrayKey($lineNumber, $data['rows']);
                        $row = $data['rows'][$row];

                        $distanceToCanvas = explode(",", $data['template']['distance_to_canvas']);

                        $font = $this->font->load($options['font'], 'font_name');

                        $fontsPath = $mediaDirectory->getAbsolutePath() . self::FONTS_FOLDER . DIRECTORY_SEPARATOR;

                        $draw->setFont($fontsPath . $font->getData('font_file'));
                        $draw->setTextAlignment(constant('\Imagick::ALIGN_' . strtoupper($row->getData('text_alignment'))));
                        $draw->setFontSize($row['font_size']);
                        $draw->annotation($distanceToCanvas[1] + $row['left'], $distanceToCanvas[0] + $row['top'], $option);
                        $im->drawImage($draw);
                    }
                }

                if (!\is_dir($mediaDirectory->getAbsolutePath() . self::PREVIEW_CACHE_FOLDER . DIRECTORY_SEPARATOR)) {
                    \mkdir($mediaDirectory->getAbsolutePath() . self::PREVIEW_CACHE_FOLDER . DIRECTORY_SEPARATOR);
                }

                $im->writeImage($cacheImagePath);

                $resultJsonFactory->setData(['image' => base64_encode($im->getImageBlob())]);
            }
        } else {
            $resultJsonFactory->setData([
                'success' => false,
                'message' => __('No SKU Passed')
            ]);
        }

        return $resultJsonFactory;
    }


    /**
     * @param $index
     * @param $rows
     * @return int
     */
    public function getTemplateRowArrayKey($index, $rows)
    {
        return (int)array_keys($rows)[$index-1];
    }

    /**
     * @param $sku
     * @param $rowCount
     * @return array|null
     */
    public function getTemplateByRowCount($sku, $rowCount)
    {
        $templates = $this->imageTemplate->getCollection()->addFieldToFilter('template_id', $sku);

        foreach ($templates->getItems() as $template) {

            if ($template->getData('row_count') == $rowCount) {
                $rows = $this->imageRow->getCollection()->addFieldToFilter('parent_template_id', $template->getId());
                return ['template' => $template, 'rows' => $rows->getItems()];
            }
        }

        return null;
    }

    /**
     * @param $options
     * @return int
     */
    protected function getLineTotal($options)
    {
        $count = 0;
        foreach ($options as $key => $value) {
            if (substr($key, 0, 5) == "line_" && $value !== '') {
                $count++;
            }
        }

        return $count;
    }

    /**
     * @param $post
     * @return array
     */
    protected function extractOptions($post)
    {
        $options = [];

        foreach (\json_decode($post, true) as $item) {
            if (substr($item['name'], 0, 7) == 'options' && isset($item['label'])) {
                $options[str_replace(" ", "_", strtolower($item['label']))] = $item['value'];
            }
        }

        return $options;
    }
}